var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import * as React from 'react';
import PeriqlesField from './PeriqlesField';
import { introspect } from './functions';
import { commitMutation } from 'react-relay';
import '../periqles.css';
var useState = React.useState, useEffect = React.useEffect;
var PeriqlesForm = function (_a) {
    var environment = _a.environment, mutationName = _a.mutationName, mutationGQL = _a.mutationGQL, useMutation = _a.useMutation, specifications = _a.specifications, _b = _a.args, args = _b === void 0 ? {} : _b, callbacks = _a.callbacks;
    var _c = useState({}), formState = _c[0], setFormState = _c[1];
    var _d = useState([]), fields = _d[0], setFields = _d[1];
    useEffect(function () {
        introspect(mutationName, setFields, args);
    }, [mutationName]);
    // HANDLERS
    var handleSubmit = function (e, fields) {
        if (e.key === 'Enter' || e.type === 'click') {
            e.preventDefault(); // prevent page refesh
        }
        // validate non-null text fields
        var fieldNames = Object.keys(formState);
        var _loop_1 = function (i) {
            var fieldObj = fields.filter(function (fieldObj) { return fieldObj.name === fieldNames[i]; })[0];
            if (fieldObj.required && formState[fieldNames[i]] === '') {
                window.alert("The following field is required: " + fieldObj.label);
                return { value: void 0 };
            }
        };
        for (var i = 0; i < fieldNames.length; i += 1) {
            var state_1 = _loop_1(i);
            if (typeof state_1 === "object")
                return state_1.value;
        }
        var input = __assign(__assign({}, formState), args);
        var variables = {
            input: input,
        };
        if (environment) {
            // relay commit method
            commitMutation(environment, {
                mutation: mutationGQL,
                variables: variables,
                onCompleted: function (response, errors) {
                    if (callbacks === null || callbacks === void 0 ? void 0 : callbacks.onSuccess)
                        callbacks.onSuccess(response);
                    setFormState({});
                },
                onError: function (err) {
                    if (callbacks === null || callbacks === void 0 ? void 0 : callbacks.onFailure)
                        callbacks.onFailure(err);
                },
            });
        }
        else {
            // apollo commit method
            // actual invocation of addUser useMutation mutate function; if passing variables must be passed inside of an object
            useMutation({ variables: variables })
                .then(function (response) {
                if (callbacks === null || callbacks === void 0 ? void 0 : callbacks.onSuccess)
                    callbacks.onSuccess(response); // useMutation mutate function returns a promise of mutation result
                setFormState({});
            })
                .catch(function (err) {
                if (callbacks === null || callbacks === void 0 ? void 0 : callbacks.onFailure)
                    callbacks.onFailure(err); // if onFailure callback provided, invoke on useMutation mutate function promise error
            });
        }
    };
    var handleChange = function (e) {
        var _a = e.target, name = _a.name, value = _a.value, type = _a.type;
        var useValue = value;
        // type-coerce values from number input elements before storing in state
        if (type === 'number' && typeof value !== 'number') {
            useValue -= 0;
        }
        var newState = Object.assign({}, formState);
        newState[name] = useValue;
        setFormState(newState);
    };
    var renderFields = function (fields) {
        return fields.map(function (field, index) {
            var specs = specifications
                ? specifications.fields[field.name]
                : undefined;
            return (React.createElement(PeriqlesField, { key: "Periqles" + mutationName + "Field" + index, field: field, specs: specs, formState: formState, setFormState: setFormState, handleChange: handleChange }));
        });
    };
    return (React.createElement("form", { className: "PeriqlesForm", "aria-labelledby": "form", onSubmit: function (e) { return handleSubmit(e, fields); } },
        specifications && specifications.header && React.createElement("h2", null, specifications.header),
        fields.length ? renderFields(fields) : React.createElement("p", null, "Loading form..."),
        React.createElement("button", { className: "periqles-submit", onClick: function (e) { return handleSubmit(e, fields); } }, "Submit")));
};
export default PeriqlesForm;
//# sourceMappingURL=PeriqlesForm.js.map